Package: zoo
Version: 1.9-0
Date: 2026-07-30
Title: S3 Infrastructure for Regular and Irregular Time Series (Z's
        Ordered Observations)
Authors@R: c(person(given = "Achim", family = "Zeileis", role = c("aut", "cre"), email = "Achim.Zeileis@R-project.org", comment = c(ORCID = "0000-0003-0918-3766")),
             person(given = "Gabor", family = "Grothendieck", role = "aut", email = "ggrothendieck@gmail.com"),
             person(given = c("Jeffrey", "A."), family = "Ryan", role = "aut", email = "jeff.a.ryan@gmail.com"),
             person(given = c("Joshua", "M."), family = "Ulrich", role = "ctb", email = "josh.m.ulrich@gmail.com"),
             person(given = "Felix", family = "Andrews", role = "ctb", email = "felix@nfrac.org"))
Description: An S3 class with methods for working with regular and irregular time series.
             The class stores data as numeric vectors/matrices (or factors) along with a time index of arbitrary class
             (including numeric, Date, POSIXct, chron, yearmon, yearqtr, etc.). Functions and methods
             are consistent with the ts class and base R and also extend standard generics. Tools include:
             Data import/export, coercion, visualization (with base R, 'ggplot2', 'lattice', 'tinyplot'), alignment and merging,
             aggregation, lags and subsets, rolling analytics, and time-based interpolation/filling.
             The design is introduced in Zeileis and Grothendieck (2005) <doi:10.18637/jss.v014.i06>.
Depends: R (>= 3.1.0), stats
Suggests: AER, coda, chron, ggplot2 (>= 3.5.0), knitr, mondate, scales,
        stinepack, strucchange, timeDate, timeSeries (>= 4021.104),
        tinyplot, tis, tseries, xts
Imports: utils, graphics, grDevices, lattice (>= 0.20-27)
License: GPL-2 | GPL-3
VignetteBuilder: knitr
URL: https://zeileis.codeberg.page/zoo/
BugReports: https://codeberg.org/zeileis/zoo/issues
NeedsCompilation: yes
Packaged: 2026-07-30 22:27:31 UTC; zeileis
Author: Achim Zeileis [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0918-3766>),
  Gabor Grothendieck [aut],
  Jeffrey A. Ryan [aut],
  Joshua M. Ulrich [ctb],
  Felix Andrews [ctb]
Maintainer: Achim Zeileis <Achim.Zeileis@R-project.org>
Repository: CRAN
Date/Publication: 2026-07-31 06:00:02 UTC
